<?php
/**
 * Orders class
 *
 * @link       https://www.fredericgilles.net/fg-prestashop-to-woocommerce/
 * @since      2.0.0
 *
 * @package    FG_Prestashop_to_WooCommerce_Premium
 * @subpackage FG_Prestashop_to_WooCommerce_Premium/admin
 */

if ( !class_exists('FG_Prestashop_to_WooCommerce_Orders', false) ) {

	/**
	 * Orders class
	 *
	 * @package    FG_Prestashop_to_WooCommerce_Premium
	 * @subpackage FG_Prestashop_to_WooCommerce_Premium/admin
	 * @author     Frédéric GILLES
	 */
	class FG_Prestashop_to_WooCommerce_Orders {

		private $customers = array();
		private $products_ids = array();
		private $shipment_methods = array();

		/**
		 * Initialize the class and set its properties.
		 *
		 * @since    2.0.0
		 * @param    object    $plugin       Admin plugin
		 */
		public function __construct( $plugin ) {

			$this->plugin = $plugin;

		}

		/**
		 * Reset the Prestashop last imported order ID
		 *
		 */
		public function reset_orders() {
			update_option('fgp2wc_last_order_id', 0);
		}
		
		/**
		 * Update the number of total elements found in PrestaShop
		 * 
		 * @since 3.0.0
		 * 
		 * @param int $count Number of total elements
		 * @return int Number of total elements
		 */
		public function get_total_elements_count($count) {
			if ( !isset($this->plugin->premium_options['skip_orders']) || !$this->plugin->premium_options['skip_orders'] ) {
				$count += $this->get_orders_count();
			}
			return $count;
		}

		/**
		 * Get the number of PrestaShop orders
		 * 
		 * @since 3.0.0
		 * 
		 * @return int Number of orders
		 */
		private function get_orders_count() {
			$prefix = $this->plugin->plugin_options['prefix'];
			$sql = "
				SELECT COUNT(*) AS nb
				FROM ${prefix}orders
			";
			$result = $this->plugin->prestashop_query($sql);
			$orders_count = isset($result[0]['nb'])? $result[0]['nb'] : 0;
			return $orders_count;
		}

		/**
		 * Import the PrestaShop orders
		 *
		 */
		public function import_orders() {
			if ( isset($this->plugin->premium_options['skip_orders']) && $this->plugin->premium_options['skip_orders'] ) {
				return;
			}
			if ( $this->plugin->import_stopped() ) {
				return;
			}
			$this->plugin->log(__('Importing orders...', $this->plugin->get_plugin_name()));
			$imported_orders_count = 0;
			
			$this->customers = $this->plugin->get_imported_customers();
			$this->products_ids = $this->plugin->get_woocommerce_products();
			$this->shipment_methods = $this->get_shipments_methods();

			do {
				if ( $this->plugin->import_stopped() ) {
					break;
				}
				$orders = $this->get_orders($this->plugin->chunks_size);
				$orders_count = count($orders);
				foreach ( $orders as $order ) {
					$order_id = $this->import_order($order);
					if ( !empty($order_id) ) {
						$imported_orders_count++;
					}
				}
				
				$this->plugin->progressbar->increment_current_count($orders_count);
				
			} while ( ($orders != null) && ($orders_count > 0) );
			$this->plugin->display_admin_notice(sprintf(_n('%d order imported', '%d orders imported', $imported_orders_count, $this->plugin->get_plugin_name()), $imported_orders_count));
		}

		/**
		 * Import an order
		 * 
		 * @since 3.52.0
		 * 
		 * @param array $order Order
		 * @return int Order ID
		 */
		private function import_order($order) {
			global $wpdb;

			// Order status
			if ( $order['current_state'] == 0 ) {
				// Look into the history table if the current_state is 0 (PrestaShop 1.4)
				$last_history = $this->get_order_history($order['id_order']);
				if ( !empty($last_history) ) {
					$current_state = $last_history['id_order_state'];
				} else {
					$current_state = 0;
				}
			} else {
				$current_state = $order['current_state'];
			}
			$order_status = $this->map_order_status($current_state);

			$order_date = $order['date_add'];

			// Insert the post
			$new_post = array(
				'post_date'			=> $order_date,
				'post_title'		=> 'Order &ndash;' . $order_date,
				'post_excerpt'		=> '',
				'post_status'		=> $order_status,
				'ping_status'		=> 'closed',
				'post_type'			=> 'shop_order',
			);

			$order_id = wp_insert_post($new_post);

			if ( $order_id ) {
				add_post_meta($order_id, '_fgp2wc_old_order_id', $order['id_order']);

				$user_id = isset($this->customers[$order['id_customer']])? $this->customers[$order['id_customer']] : 0;
				$customer = get_user_by('id', $user_id);

				// Billing address
				$billing_address = $this->get_address($order['id_address_invoice']);
				if ( !empty($billing_address) ) {
					add_post_meta($order_id, '_billing_country', $billing_address['country'], true);
					add_post_meta($order_id, '_billing_first_name', $billing_address['firstname'], true);
					add_post_meta($order_id, '_billing_last_name', $billing_address['lastname'], true);
					add_post_meta($order_id, '_billing_company', $billing_address['company'], true);
					add_post_meta($order_id, '_billing_address_1', $billing_address['address1'], true);
					add_post_meta($order_id, '_billing_address_2', $billing_address['address2'], true);
					add_post_meta($order_id, '_billing_postcode', $billing_address['postcode'], true);
					add_post_meta($order_id, '_billing_city', $billing_address['city'], true);
					add_post_meta($order_id, '_billing_state', $billing_address['state'], true);
					if ( $customer ) {
						add_post_meta($order_id, '_billing_email', $customer->user_email, true);
					}
					add_post_meta($order_id, '_billing_phone', $billing_address['phone'], true);
				}

				// Shipping address
				$shipping_address = $this->get_address($order['id_address_delivery']);
				if ( !empty($shipping_address) ) {
					add_post_meta($order_id, '_shipping_country', $shipping_address['country'], true);
					add_post_meta($order_id, '_shipping_first_name', $shipping_address['firstname'], true);
					add_post_meta($order_id, '_shipping_last_name', $shipping_address['lastname'], true);
					add_post_meta($order_id, '_shipping_company', $shipping_address['company'], true);
					add_post_meta($order_id, '_shipping_address_1', $shipping_address['address1'], true);
					add_post_meta($order_id, '_shipping_address_2', $shipping_address['address2'], true);
					add_post_meta($order_id, '_shipping_postcode', $shipping_address['postcode'], true);
					add_post_meta($order_id, '_shipping_city', $shipping_address['city'], true);
					add_post_meta($order_id, '_shipping_state', $shipping_address['state'], true);
				}

				add_post_meta($order_id, '_payment_method', $order['payment'], true);
				add_post_meta($order_id, '_payment_method_title', $order['payment'], true);
				add_post_meta($order_id, '_order_shipping', $order['total_shipping'], true);
				add_post_meta($order_id, '_order_discount', $order['total_discounts'], true);
				add_post_meta($order_id, '_cart_discount', 0, true);
				add_post_meta($order_id, '_order_tax', $order['total_tax'], true);
				add_post_meta($order_id, '_order_shipping_tax', $order['total_shipping_tax'], true);
				add_post_meta($order_id, '_order_total', $order['total_paid'], true);
				add_post_meta($order_id, '_order_key', $order['reference'], true);
				add_post_meta($order_id, '_customer_user', $user_id, true);
				add_post_meta($order_id, '_order_currency', $order['currency'], true);
				add_post_meta($order_id, '_prices_include_tax', 'no', true);
				add_post_meta($order_id, '_customer_ip_address', '', true);
				add_post_meta($order_id, '_customer_user_agent', '', true);
				add_post_meta($order_id, '_recorded_sales', 'yes', true);
				add_post_meta($order_id, '_recorded_coupon_usage_counts', 'yes', true);

				// Order items
				$order_items = $this->get_order_items($order['id_order']);
				foreach ( $order_items as $order_item ) {
					$this->add_order_item($order_id, $order['reference'], $order_item, $user_id, $order_date);
				}

				// Shipping
				$shipment_method = isset($this->shipment_methods[$order['id_carrier']])? $this->shipment_methods[$order['id_carrier']] : '';
				if ( $wpdb->insert($wpdb->prefix . 'woocommerce_order_items', array(
					'order_item_name'	=> $shipment_method,
					'order_item_type'	=> 'shipping',
					'order_id'			=> $order_id,
				)) ) {
					$wc_order_item_id = $wpdb->insert_id;
					$this->add_wc_order_itemmeta($wc_order_item_id, 'method_id', 0);
					$this->add_wc_order_itemmeta($wc_order_item_id, 'cost', $order['total_shipping']);
				}

				// Taxes
				if ( $wpdb->insert($wpdb->prefix . 'woocommerce_order_items', array(
					'order_item_name'	=> 'Tax',
					'order_item_type'	=> 'tax',
					'order_id'			=> $order_id,
				)) ) {
					$wc_order_item_id = $wpdb->insert_id;
					$this->add_wc_order_itemmeta($wc_order_item_id, 'rate_id', 0);
					$this->add_wc_order_itemmeta($wc_order_item_id, 'label', 'Tax');
					$this->add_wc_order_itemmeta($wc_order_item_id, 'compound', 0);
					$this->add_wc_order_itemmeta($wc_order_item_id, 'tax_amount', $order['total_tax']);
					$this->add_wc_order_itemmeta($wc_order_item_id, 'shipping_tax_amount', $order['total_shipping_tax']);
				}
				
				// Hook for doing other actions after inserting the order
				do_action('fgp2wc_post_insert_order', $order_id, $order);
			}

			// Increment the PrestaShop last imported order ID
			update_option('fgp2wc_last_order_id', $order['id_order']);
			
			return $order_id;
		}
		
		/**
		 * Mapping between PrestaShop and WooCommerce status
		 *
		 * @param string $ps_status PrestaShop order status
		 * @return string WooCommerce order status
		 */
		private function map_order_status($ps_status) {
			switch ( $ps_status ) {
				case '1': // waiting cheque
				case '10': // waiting bank wire
				case '11': // waiting Paypal
					$status = 'wc-pending'; break;
				case '2': // payment accepted
				case '3': // ongoing preparation
				case '12': // remote payment accepted
					$status = 'wc-processing'; break;
				case '9': // out of stock
					$status = 'wc-on-hold'; break;
				case '6': // cancelled
					$status = 'wc-cancelled'; break;
				case '8': // payment error
					$status = 'wc-failed'; break;
				case '7': // refund
					$status = 'wc-refunded'; break;
				case '4': // ongoing shipping
				case '5': // shipped
					$status = 'wc-completed'; break;
				default:
					$status = 'wc-pending'; break;
			}
			$status = apply_filters('fgp2wc_map_order_status', $status, $ps_status);
			return $status;
		}

		/**
		 * Add an order item row into the order
		 * 
		 * @since 3.52.0
		 * 
		 * @param int $order_id Order ID
		 * @param string $order_key Order key
		 * @param array $order_item Order item
		 * @param int $user_id Customer ID
		 * @param date $order_date Order date
		 * @return int $wc_order_item_id Order item ID
		 */
		private function add_order_item($order_id, $order_key, $order_item, $user_id, $order_date) {
			global $wpdb;
			
			if ( $wpdb->insert($wpdb->prefix . 'woocommerce_order_items', array(
				'order_item_name'	=> $order_item['product_name'],
				'order_item_type'	=> 'line_item',
				'order_id'			=> $order_id,
			)) ) {
				$wc_order_item_id = $wpdb->insert_id;
				$product_id = isset($this->products_ids[$order_item['product_id']])? $this->products_ids[$order_item['product_id']]: 0;
				$variation_id = $this->get_variation_id($order_item['product_attribute_id']);
				$line_total = $order_item['product_price'];
				$line_tax = $order_item['product_tax'];
				$this->add_wc_order_itemmeta($wc_order_item_id, '_qty', $order_item['product_quantity']);
				$this->add_wc_order_itemmeta($wc_order_item_id, '_tax_class', $order_item['tax_name']);
				$this->add_wc_order_itemmeta($wc_order_item_id, '_product_id', $product_id);
				$this->add_wc_order_itemmeta($wc_order_item_id, '_variation_id', $variation_id);
				$this->add_wc_order_itemmeta($wc_order_item_id, '_line_subtotal', $line_total);
				$this->add_wc_order_itemmeta($wc_order_item_id, '_line_total', $line_total);
				$this->add_wc_order_itemmeta($wc_order_item_id, '_line_tax', $line_tax);
				$this->add_wc_order_itemmeta($wc_order_item_id, '_line_subtotal_tax', $line_tax);
				$this->add_wc_order_itemmeta($wc_order_item_id, '_line_tax_data', array(
					'total' => array(1 => $line_tax),
					'subtotal' => array(1 => $line_tax),
				));

				// Downloadable products
				if ( !empty($order_item['download_hash']) ) {
					// Create downloadable product permission rows
					$customer = get_user_by('id', $user_id);
					$customer_email = isset($customer->user_email)? $customer->user_email : '';
					$this->add_wc_downloadable_product_permission($order_item, $order_id, $order_date, $order_key, $product_id, $user_id, $customer_email);
				}

				// Hook for doing other actions after inserting the order item
				do_action('fgp2wc_post_insert_order_item', $wc_order_item_id, $order_item);
			}
			
			return $wc_order_item_id;
		}
		
		/**
		 * Get PrestaShop orders
		 * 
		 * @param int $limit Number of articles max
		 * @return array Orders
		 */
		private function get_orders($limit=1000) {
			$orders = array();

			$last_prestashop_order_id = (int)get_option('fgp2wc_last_order_id'); // to restore the import where it left

			$prefix = $this->plugin->plugin_options['prefix'];

			if ( version_compare($this->plugin->prestashop_version, '1.3', '<') ) {
				// PrestaShop 1.2 and less
				$sql = "
					SELECT o.id_order, '' AS reference, o.id_carrier, o.id_customer, o.id_address_delivery, o.id_address_invoice, 0 AS current_state, o.payment, o.total_paid, o.total_paid - o.total_products - o.total_shipping - o.total_wrapping AS total_tax, o.total_products, o.total_shipping, 0 AS total_shipping_tax, o.total_wrapping, o.total_discounts, o.secure_key, o.date_add,
					c.iso_code AS currency
					FROM ${prefix}orders o
					LEFT JOIN ${prefix}currency c ON c.id_currency = o.id_currency
					WHERE o.id_order > '$last_prestashop_order_id'
					ORDER BY o.id_order
					LIMIT $limit
				";
			} elseif ( version_compare($this->plugin->prestashop_version, '1.5', '<') ) {
				// PrestaShop 1.4
				$sql = "
					SELECT o.id_order, '' AS reference, o.id_carrier, o.id_customer, o.id_address_delivery, o.id_address_invoice, 0 AS current_state, o.payment, o.total_paid, o.total_products_wt - o.total_products AS total_tax, o.total_products, o.total_shipping, 0 AS total_shipping_tax, o.total_wrapping, o.total_discounts, o.secure_key, o.date_add,
					c.iso_code AS currency
					FROM ${prefix}orders o
					LEFT JOIN ${prefix}currency c ON c.id_currency = o.id_currency
					WHERE o.id_order > '$last_prestashop_order_id'
					ORDER BY o.id_order
					LIMIT $limit
				";
			} else {
				// PrestaShop 1.5+
				$sql = "
					SELECT o.id_order, o.reference, o.id_carrier, o.id_customer, o.id_address_delivery, o.id_address_invoice, o.current_state, o.payment, o.total_paid, o.total_paid_tax_incl - o.total_paid_tax_excl AS total_tax, o.total_products, o.total_shipping, o.total_shipping_tax_incl - o.total_shipping_tax_excl AS total_shipping_tax, o.total_wrapping, o.total_discounts, o.secure_key, o.date_add,
					c.iso_code AS currency
					FROM ${prefix}orders o
					LEFT JOIN ${prefix}currency c ON c.id_currency = o.id_currency
					WHERE o.id_order > '$last_prestashop_order_id'
					ORDER BY o.id_order
					LIMIT $limit
				";
			}
			$orders = $this->plugin->prestashop_query($sql);

			return $orders;
		}

		/**
		 * Get PrestaShop order items
		 * 
		 * @param int $order_id Order ID
		 * @return array Order items
		 */
		private function get_order_items($order_id) {
			$order_items = array();

			$prefix = $this->plugin->plugin_options['prefix'];
			if ( version_compare($this->plugin->prestashop_version, '1.5', '<') ) {
				// PrestaShop 1.4
				$sql = "
					SELECT d.id_order_detail, d.id_order, d.product_id, d.product_attribute_id, d.product_name, d.product_quantity, d.tax_name, d.product_price * d.tax_rate / 100 AS product_tax, d.product_price * d.product_quantity AS product_price, d.download_hash, d.download_nb, d.download_deadline
					FROM ${prefix}order_detail d
					WHERE d.id_order = '$order_id'
					ORDER BY id_order_detail
				";
			} else {
				// PrestaShop 1.5+
				$sql = "
					SELECT d.id_order_detail, d.id_order, d.product_id, d.product_attribute_id, d.product_name, d.product_quantity, d.tax_name, d.total_price_tax_incl - d.total_price_tax_excl AS product_tax, d.total_price_tax_excl AS product_price, d.download_hash, d.download_nb, d.download_deadline
					FROM ${prefix}order_detail d
					WHERE d.id_order = '$order_id'
					ORDER BY id_order_detail
				";
			}
			$order_items = $this->plugin->prestashop_query($sql);

			return $order_items;
		}

		/**
		 * Get PrestaShop order history
		 * 
		 * @param int $order_id Order ID
		 * @return array Order history
		 */
		private function get_order_history($order_id) {
			$order_history = array();

			$prefix = $this->plugin->plugin_options['prefix'];
			$sql = "
				SELECT h.id_order_state
				FROM ${prefix}order_history h
				WHERE h.id_order = '$order_id'
				ORDER BY h.date_add DESC
				LIMIT 1
			";
			$result = $this->plugin->prestashop_query($sql);
			if ( count($result) > 0 ) {
				$order_history = $result[0];
			}

			return $order_history;
		}

		/**
		 * Get a PrestaShop address
		 * 
		 * @param int $address_id PrestaShop address ID
		 * @return array Address fields
		 */
		private function get_address($address_id) {
			$address = array();

			$prefix = $this->plugin->plugin_options['prefix'];
			$sql = "
				SELECT a.company, a.lastname, a.firstname, a.address1, a.address2, a.postcode, a.city, a.phone,
				c.iso_code AS country, s.iso_code AS state
				FROM ${prefix}address a
				LEFT JOIN ${prefix}country c ON c.id_country = a.id_country
				LEFT JOIN ${prefix}state s ON s.id_state = a.id_state
				WHERE a.id_address = '$address_id'
				LIMIT 1
			";
			$result = $this->plugin->prestashop_query($sql);
			if ( count($result) > 0 ) {
				$address = $result[0];
			}
			return $address;
		}

		/**
		 * Get PrestaShop shipment methods
		 * 
		 * @return array Shipment methods
		 */
		private function get_shipments_methods() {
			$shipment_methods = array();

			$prefix = $this->plugin->plugin_options['prefix'];
			$sql = "
				SELECT c.id_carrier, c.name
				FROM ${prefix}carrier c
			";
			$result = $this->plugin->prestashop_query($sql);
			foreach ( $result as $row ) {
				$shipment_methods[$row['id_carrier']] = $row['name'];
			}

			return $shipment_methods;
		}

		/**
		 * Get the WooCommerce variation ID
		 *
		 * @param int $product_attribute_id PrestaShop product attribute ID
		 * @return int WooCommerce variation ID
		 */
		private function get_variation_id($product_attribute_id) {
			global $wpdb;
			$variation_id = 0;

			try {
				$sql = "
					SELECT post_id
					FROM $wpdb->postmeta
					WHERE meta_key = '_fgp2wc_old_product_attribute_id'
					AND meta_value = '$product_attribute_id'
					LIMIT 1
				";
				$variation_id = $wpdb->get_var($sql);
			} catch ( PDOException $e ) {
				$this->plugin->display_admin_error(__('Error:', $this->plugin->get_plugin_name()) . $e->getMessage());
			}
			return $variation_id;
		}

		/**
		 * Add an order item meta
		 *
		 * @param int $order_item_id Order item ID
		 * @param string $meta_key Meta key
		 * @param string $meta_value Meta value
		 * @return int order item meta ID
		 */
		private function add_wc_order_itemmeta($order_item_id, $meta_key, $meta_value) {
			global $wpdb;
			if ( !is_scalar($meta_value) ) {
				$meta_value = serialize($meta_value);
			}
			if ( $wpdb->insert($wpdb->prefix . 'woocommerce_order_itemmeta', array(
				'order_item_id'	=> $order_item_id,
				'meta_key'		=> $meta_key,
				'meta_value'	=> $meta_value,
			)) ) {
				return $wpdb->insert_id;
			} else {
				return 0;
			}
		}
		
		/**
		 * Add downloadable product permission rows
		 * 
		 * @since 2.7.0
		 * 
		 * @param array $order_item Order item data
		 * @param int $order_id Order ID
		 * @param date $order_date Order creation date
		 * @param string $order_key Order key
		 * @param int $product_id Product ID
		 * @param int $user_id User ID
		 * @param string $customer_email Customer email
		 */
		private function add_wc_downloadable_product_permission($order_item, $order_id, $order_date, $order_key, $product_id, $user_id, $customer_email) {
			global $wpdb;
			$download_ids = $this->get_download_ids($product_id);
			foreach ( $download_ids as $download_id ) {
				$wpdb->insert($wpdb->prefix . 'woocommerce_downloadable_product_permissions', array(
					'download_id'			=> $download_id,
					'product_id'			=> $product_id,
					'order_id'				=> $order_id,
					'order_key'				=> $order_key,
					'user_email'			=> $customer_email,
					'user_id'				=> $user_id,
					'downloads_remaining'	=> $this->calculate_downloads_remaining($product_id, $order_item['download_nb']),
					'access_granted'		=> $order_date,
					'access_expires'		=> $order_item['download_deadline'],
					'download_count'		=> $order_item['download_nb'],
				));
			}
		}
		
		/**
		 * Get the download IDs of a product
		 * 
		 * @since 2.7.0
		 * 
		 * @param int $product_id Product ID
		 * @return array Download IDs
		 */
		private function get_download_ids($product_id) {
			$download_ids = array();
			$downloadable_files = get_post_meta($product_id, '_downloadable_files');
			if ( is_array($downloadable_files) ) {
				foreach ( $downloadable_files as $downloadable_file ) {
					$download_ids = array_merge($download_ids, array_keys($downloadable_file));
				}
			}
			return $download_ids;
		}
		
		/**
		 * Calculate the downloads remaining number
		 * 
		 * @since 2.7.0
		 * 
		 * @param int $product_id Product ID
		 * @param int $download_nb Downloads already processed
		 * @return int Downloads remaining
		 */
		private function calculate_downloads_remaining($product_id, $download_nb) {
			$downloads_remaining = '';
			$download_limit = get_post_meta($product_id, '_download_limit', true);
			if ( !empty($download_limit) ) {
				$downloads_remaining = $download_limit - $download_nb;
			}
			return $downloads_remaining;
		}
		
		/**
		 * Update the already imported orders
		 * 
		 * @since 3.22.0
		 */
		public function update_orders() {
			$this->plugin->log(__('Updating orders...', $this->plugin->get_plugin_name()));
			$updated_orders_count = 0;

			$last_update = get_option('fgp2wc_last_update');

			$orders = $this->get_updated_orders($last_update);
			foreach ( $orders as $order ) {
				$order_id = $this->get_wp_order_id_from_prestashop_id($order['id_order']);
				if ( !empty($order_id) ) {
					// Order status
					$order_status = $this->map_order_status($order['current_state']);
					// Update the order status
					wp_update_post(array(
						'ID'			=> $order_id,
						'post_status'	=> $order_status,
					));
					$updated_orders_count++;
				}
			}

			// Hook for doing other actions after all orders are updated
			do_action('fgp2wc_post_update_orders');

			$this->plugin->display_admin_notice(sprintf(_n('%d order updated', '%d orders updated', $updated_orders_count, $this->plugin->get_plugin_name()), $updated_orders_count));
		}

		/**
		 * Get the orders updated after a date
		 * 
		 * @since 3.22.0
		 * 
		 * @param date $last_update
		 */
		private function get_updated_orders($last_update) {
			$orders = array();
			$prefix = $this->plugin->plugin_options['prefix'];

			if ( version_compare($this->plugin->prestashop_version, '1.3', '<') ) {
				// PrestaShop 1.2 and less
				$sql = "
					SELECT o.id_order, '' AS reference, o.id_carrier, o.id_customer, o.id_address_delivery, o.id_address_invoice, 0 AS current_state, o.payment, o.total_paid, o.total_paid - o.total_products - o.total_shipping - o.total_wrapping AS total_tax, o.total_products, o.total_shipping, 0 AS total_shipping_tax, o.total_wrapping, o.total_discounts, o.secure_key, o.date_add,
					c.iso_code AS currency
					FROM ${prefix}orders o
					LEFT JOIN ${prefix}currency c ON c.id_currency = o.id_currency
					WHERE o.date_upd > '$last_update'
					ORDER BY o.date_upd
				";
			} elseif ( version_compare($this->plugin->prestashop_version, '1.5', '<') ) {
				// PrestaShop 1.4
				$sql = "
					SELECT o.id_order, '' AS reference, o.id_carrier, o.id_customer, o.id_address_delivery, o.id_address_invoice, 0 AS current_state, o.payment, o.total_paid, o.total_products_wt - o.total_products AS total_tax, o.total_products, o.total_shipping, 0 AS total_shipping_tax, o.total_wrapping, o.total_discounts, o.secure_key, o.date_add,
					c.iso_code AS currency
					FROM ${prefix}orders o
					LEFT JOIN ${prefix}currency c ON c.id_currency = o.id_currency
					WHERE o.date_upd > '$last_update'
					ORDER BY o.date_upd
				";
			} else {
				// PrestaShop 1.5+
				$sql = "
					SELECT o.id_order, o.reference, o.id_carrier, o.id_customer, o.id_address_delivery, o.id_address_invoice, o.current_state, o.payment, o.total_paid, o.total_paid_tax_incl - o.total_paid_tax_excl AS total_tax, o.total_products, o.total_shipping, o.total_shipping_tax_incl - o.total_shipping_tax_excl AS total_shipping_tax, o.total_wrapping, o.total_discounts, o.secure_key, o.date_add,
					c.iso_code AS currency
					FROM ${prefix}orders o
					LEFT JOIN ${prefix}currency c ON c.id_currency = o.id_currency
					WHERE o.date_upd > '$last_update'
					ORDER BY o.date_upd
				";
			}
			$orders = $this->plugin->prestashop_query($sql);

			return $orders;
		}
		
		/**
		 * Returns the imported order ID corresponding to a PrestaShop ID
		 *
		 * @since 3.22.0
		 * 
		 * @param int $ps_order_id PrestaShop order ID
		 * @return int WordPress order ID
		 */
		public function get_wp_order_id_from_prestashop_id($ps_order_id) {
			$order_id = $this->plugin->get_wp_post_id_from_meta('_fgp2wc_old_order_id', $ps_order_id);
			return $order_id;
		}

	}
}
